@@ -1,4 +1,4 @@
-/* =======================================================================
+/* ==============================================================
  *
  * Fixbot
  *
@@ -10,6 +10,10 @@
 
 #define MZ2_fixbot_BLASTER_1 MZ2_HOVER_BLASTER_1
 
+#define FIXBOT_MAX_STUCK_FRAMES		10
+#define FIXBOT_GOAL_TIMEOUT			15
+#define FIXBOT_WELD_GOAL_TIMEOUT	15
+
 qboolean visible(edict_t *self, edict_t *other);
 qboolean infront(edict_t *self, edict_t *other);
 
@@ -178,8 +182,7 @@ make_bot_goal(edict_t *self)
 
 	ent->think = bot_goal_think;
 	ent->nextthink = level.time + FRAMETIME;
-
-	gi.linkentity(ent);
+	ent->touch_debounce_time = level.time + FIXBOT_GOAL_TIMEOUT;
 
 	return ent;
 }
@@ -198,9 +201,9 @@ landing_goal(edict_t *self)
 	}
 
 	ent = make_bot_goal(self);
-
 	VectorSet(ent->mins, -32, -32, -24);
 	VectorSet(ent->maxs, 32, 32, 24);
+	gi.linkentity(ent);
 
 	AngleVectors(self->s.angles, forward, right, up);
 	VectorMA(self->s.origin, 32, forward, end);
@@ -210,6 +213,7 @@ landing_goal(edict_t *self)
 			end, self, MASK_MONSTERSOLID);
 
 	VectorCopy(tr.endpos, ent->s.origin);
+	gi.linkentity(ent);
 
 	self->goalentity = self->enemy = ent;
 	self->monsterinfo.currentmove = &fixbot_move_landing;
@@ -232,6 +236,7 @@ takeoff_goal(edict_t *self)
 
 	VectorSet(ent->mins, -32, -32, -24);
 	VectorSet(ent->maxs, 32, 32, 24);
+	gi.linkentity(ent);
 
 	AngleVectors(self->s.angles, forward, right, up);
 	VectorMA(self->s.origin, 32, forward, end);
@@ -241,6 +246,7 @@ takeoff_goal(edict_t *self)
 			end, self, MASK_MONSTERSOLID);
 
 	VectorCopy(tr.endpos, ent->s.origin);
+	gi.linkentity(ent);
 
 	self->goalentity = self->enemy = ent;
 	self->monsterinfo.currentmove = &fixbot_move_takeoff;
@@ -291,7 +297,7 @@ void
 roam_goal(edict_t *self)
 {
 	trace_t tr;
-	vec3_t forward, right, up;
+	vec3_t forward;
 	vec3_t end;
 	edict_t *ent;
 	vec3_t dang;
@@ -304,11 +310,7 @@ roam_goal(edict_t *self)
 		return;
 	}
 
-	whichvec[0] = 0;
-	whichvec[1] = 0;
-	whichvec[2] = 0;
-
-	ent = make_bot_goal(self);
+	VectorClear(whichvec);
 
 	oldlen = 0;
 
@@ -325,13 +327,13 @@ roam_goal(edict_t *self)
 			dang[YAW] -= 30 * (i - 6);
 		}
 
-		AngleVectors(dang, forward, right, up);
+		AngleVectors(dang, forward, NULL, NULL);
 		VectorMA(self->s.origin, 8192, forward, end);
 
 		tr = gi.trace(self->s.origin, NULL, NULL, end, self, MASK_SHOT);
 
 		VectorSubtract(self->s.origin, tr.endpos, vec);
-		len = VectorNormalize(vec);
+		len = VectorLength(vec);
 
 		if (len > oldlen)
 		{
@@ -340,7 +342,10 @@ roam_goal(edict_t *self)
 		}
 	}
 
+	ent = make_bot_goal(self);
 	VectorCopy(whichvec, ent->s.origin);
+	gi.linkentity(ent);
+
 	self->goalentity = self->enemy = ent;
 
 	self->monsterinfo.currentmove = &fixbot_move_turn;
@@ -350,84 +355,102 @@ void
 use_scanner(edict_t *self)
 {
 	edict_t *ent = NULL;
-	float radius = 1024;
 	vec3_t vec;
-	int len;
 
   	if (!self)
 	{
 		return;
 	}
 
-	while ((ent = findradius(ent, self->s.origin, radius)) != NULL)
+	if (self->fly_sound_debounce_time < level.time &&
+		strcmp(self->goalentity->classname, "object_repair") != 0)
 	{
-		if (ent->health >= 100)
+		while ((ent = findradius(ent, self->s.origin, 1024)) != NULL)
 		{
-			if (strcmp(ent->classname, "object_repair") == 0)
+			if (strcmp(ent->classname, "object_repair") != 0)
 			{
-				if (visible(self, ent))
-				{
-					/* remove the old one */
-					if (strcmp(self->goalentity->classname, "bot_goal") == 0)
-					{
-						self->goalentity->nextthink = level.time + 0.1;
-						self->goalentity->think = G_FreeEdict;
-					}
-
-					self->goalentity = self->enemy = ent;
-
-					VectorSubtract(self->s.origin, self->goalentity->s.origin, vec);
-					len = VectorNormalize(vec);
-
-					if (len < 32)
-					{
-						self->monsterinfo.currentmove = &fixbot_move_weld_start;
-						return;
-					}
-
-					return;
-				}
+				continue;
 			}
+
+			if (ent->health < 100)
+			{
+				continue;
+			}
+
+			if (!visible(self, ent))
+			{
+				continue;
+			}
+
+			/* remove the old one */
+			if (strcmp(self->goalentity->classname, "bot_goal") == 0)
+			{
+				self->goalentity->nextthink = level.time + 0.1;
+				self->goalentity->think = G_FreeEdict;
+			}
+
+			self->goalentity = self->enemy = ent;
+
+			break;
 		}
 	}
 
-	VectorSubtract(self->s.origin, self->goalentity->s.origin, vec);
-	len = VectorLength(vec);
-
-	if (len < 32)
+	if (strcmp(self->goalentity->classname, "object_repair") == 0)
 	{
-		if (strcmp(self->goalentity->classname, "object_repair") == 0)
+		VectorSubtract(self->s.origin, self->goalentity->s.origin, vec);
+
+		if (VectorLength(vec) < 56)
 		{
 			self->monsterinfo.currentmove = &fixbot_move_weld_start;
+			return;
 		}
-		else
+	}
+	else if (strcmp(self->goalentity->classname, "bot_goal") == 0)
+	{
+		VectorSubtract(self->s.origin, self->goalentity->s.origin, vec);
+
+		if (self->goalentity->touch_debounce_time < level.time || VectorLength(vec) < 32)
 		{
 			self->goalentity->nextthink = level.time + 0.1;
 			self->goalentity->think = G_FreeEdict;
 			self->goalentity = self->enemy = NULL;
-			self->monsterinfo.currentmove = &fixbot_move_stand;
-		}
 
-		return;
+			self->monsterinfo.currentmove = &fixbot_move_stand;
+
+			return;
+		}
 	}
 
 	VectorSubtract(self->s.origin, self->s.old_origin, vec);
-	len = VectorLength(vec);
 
-	/* bot is stuck get new goalentity */
-	if (len == 0)
+	if (VectorLength(vec) == 0)
 	{
-		if (strcmp(self->goalentity->classname, "object_repair") == 0)
-		{
-			self->monsterinfo.currentmove = &fixbot_move_stand;
-		}
-		else
+		self->count++;
+	}
+	else
+	{
+		self->count = 0;
+	}
+
+	if (self->count > FIXBOT_MAX_STUCK_FRAMES)
+	{
+		/* bot is stuck, get new goalentity */
+
+		self->count = 0;
+
+		if (strcmp(self->goalentity->classname, "bot_goal") == 0)
 		{
 			self->goalentity->nextthink = level.time + 0.1;
 			self->goalentity->think = G_FreeEdict;
 			self->goalentity = self->enemy = NULL;
-			self->monsterinfo.currentmove = &fixbot_move_stand;
 		}
+		else if (strcmp(self->goalentity->classname, "object_repair") == 0)
+		{
+			/* don't try to go for welding targets again for a while */
+			self->fly_sound_debounce_time = level.time + FIXBOT_WELD_GOAL_TIMEOUT;
+		}
+
+		self->monsterinfo.currentmove = &fixbot_move_stand;
 	}
 }
 
@@ -668,7 +691,7 @@ fly_vertical2(edict_t *self)
 	}
 }
 
-mframe_t fixbot_frames_landing[] = {
+static mframe_t fixbot_frames_landing[] = {
 	{ai_move, 0, NULL},
 	{ai_move, 0, fly_vertical2},
 	{ai_move, 0, fly_vertical2},
@@ -742,7 +765,7 @@ mmove_t fixbot_move_landing = {
 };
 
 /*  generic ambient stand */
-mframe_t fixbot_frames_stand[] = {
+static mframe_t fixbot_frames_stand[] = {
 	{ai_move, 0, NULL},
 	{ai_move, 0, NULL},
 	{ai_move, 0, NULL},
@@ -772,7 +795,7 @@ mmove_t fixbot_move_stand = {
    	NULL
 };
 
-mframe_t fixbot_frames_stand2[] = {
+static mframe_t fixbot_frames_stand2[] = {
 	{ai_stand, 0, NULL},
 	{ai_stand, 0, NULL},
 	{ai_stand, 0, NULL},
@@ -808,7 +831,7 @@ mmove_t fixbot_move_stand2 = {
  *  and take the object with it ( this may require a
  *  variant of liftoff and landing )
  */
-mframe_t fixbot_frames_pickup[] = {
+static mframe_t fixbot_frames_pickup[] = {
 	{ai_move, 0, NULL},
 	{ai_move, 0, NULL},
 	{ai_move, 0, NULL},
@@ -848,7 +871,7 @@ mmove_t fixbot_move_pickup = {
 };
 
 /*  generic frame to move bot */
-mframe_t fixbot_frames_roamgoal[] = {
+static mframe_t fixbot_frames_roamgoal[] = {
 	{ai_move, 0, roam_goal}
 };
 
@@ -881,7 +904,7 @@ ai_facing(edict_t *self, float dist)
 	}
 }
 
-mframe_t fixbot_frames_turn[] = {
+static mframe_t fixbot_frames_turn[] = {
 	{ai_facing, 0, NULL}
 };
 
@@ -904,7 +927,7 @@ go_roam(edict_t *self)
 }
 
 /* takeoff */
-mframe_t fixbot_frames_takeoff[] = {
+static mframe_t fixbot_frames_takeoff[] = {
 	{ai_move, 0.01, fly_vertical},
 	{ai_move, 0.01, fly_vertical},
 	{ai_move, 0.01, fly_vertical},
@@ -932,7 +955,7 @@ mmove_t fixbot_move_takeoff = {
 };
 
 /* findout what this is */
-mframe_t fixbot_frames_paina[] = {
+static mframe_t fixbot_frames_paina[] = {
 	{ai_move, 0, NULL},
 	{ai_move, 0, NULL},
 	{ai_move, 0, NULL},
@@ -949,7 +972,7 @@ mmove_t fixbot_move_paina = {
 };
 
 /* findout what this is */
-mframe_t fixbot_frames_painb[] = {
+static mframe_t fixbot_frames_painb[] = {
 	{ai_move, 0, NULL},
 	{ai_move, 0, NULL},
 	{ai_move, 0, NULL},
@@ -972,7 +995,7 @@ mmove_t fixbot_move_painb = {
  *  call a generic painsound
  *  some spark effects
  */
-mframe_t fixbot_frames_pain3[] = {
+static mframe_t fixbot_frames_pain3[] = {
 	{ai_move, -1, NULL}
 };
 
@@ -989,7 +1012,7 @@ mmove_t fixbot_move_pain3 = {
  *  ( may need second land if the
  *  bot is releasing jib into jib vat )
  */
-mframe_t fixbot_frames_land[] = {
+static mframe_t fixbot_frames_land[] = {
 	{ai_move, 0, NULL}
 };
 
@@ -1011,7 +1034,7 @@ ai_movetogoal(edict_t *self, float dist)
 	M_MoveToGoal(self, dist);
 }
 
-mframe_t fixbot_frames_forward[] = {
+static mframe_t fixbot_frames_forward[] = {
 	{ai_movetogoal, 5, use_scanner}
 };
 
@@ -1022,7 +1045,7 @@ mmove_t fixbot_move_forward = {
    	NULL
 };
 
-mframe_t fixbot_frames_walk[] = {
+static mframe_t fixbot_frames_walk[] = {
 	{ai_walk, 5, NULL}
 };
 
@@ -1033,7 +1056,7 @@ mmove_t fixbot_move_walk = {
 	NULL
 };
 
-mframe_t fixbot_frames_run[] = {
+static mframe_t fixbot_frames_run[] = {
 	{ai_run, 10, NULL}
 };
 
@@ -1044,7 +1067,7 @@ mmove_t fixbot_move_run = {
    	NULL
 };
 
-mframe_t fixbot_frames_death1[] = {
+static mframe_t fixbot_frames_death1[] = {
 	{ai_move, 0, NULL}
 
 };
@@ -1055,7 +1078,7 @@ mmove_t fixbot_move_death1 = {
    	fixbot_dead
 };
 
-mframe_t fixbot_frames_backward[] = {
+static mframe_t fixbot_frames_backward[] = {
 	{ai_move, 0, NULL}
 };
 
@@ -1066,7 +1089,7 @@ mmove_t fixbot_move_backward = {
    	NULL
 };
 
-mframe_t fixbot_frames_start_attack[] = {
+static mframe_t fixbot_frames_start_attack[] = {
 	{ai_charge, 0, NULL}
 };
 
@@ -1077,7 +1100,7 @@ mmove_t fixbot_move_start_attack = {
    	fixbot_attack
 };
 
-mframe_t fixbot_frames_attack1[] = {
+static mframe_t fixbot_frames_attack1[] = {
 	{ai_charge, 0, NULL},
 	{ai_charge, 0, NULL},
 	{ai_charge, 0, NULL},
@@ -1195,7 +1218,7 @@ fixbot_fire_laser(edict_t *self)
 	}
 }
 
-mframe_t fixbot_frames_laserattack[] = {
+static mframe_t fixbot_frames_laserattack[] = {
 	{ai_charge, 0, fixbot_fire_laser},
 	{ai_charge, 0, fixbot_fire_laser},
 	{ai_charge, 0, fixbot_fire_laser},
@@ -1213,7 +1236,7 @@ mmove_t fixbot_move_laserattack = {
 
 /* need to get forward translation
    data for the charge attack */
-mframe_t fixbot_frames_attack2[] = {
+static mframe_t fixbot_frames_attack2[] = {
 	{ai_charge, 0, NULL},
 	{ai_charge, 0, NULL},
 	{ai_charge, 0, NULL},
@@ -1308,7 +1331,7 @@ ai_move2(edict_t *self, float dist)
 	M_ChangeYaw(self);
 }
 
-mframe_t fixbot_frames_weld_start[] = {
+static mframe_t fixbot_frames_weld_start[] = {
 	{ai_move2, 0, NULL},
 	{ai_move2, 0, NULL},
 	{ai_move2, 0, NULL},
@@ -1328,7 +1351,7 @@ mmove_t fixbot_move_weld_start = {
    	NULL
 };
 
-mframe_t fixbot_frames_weld[] = {
+static mframe_t fixbot_frames_weld[] = {
 	{ai_move2, 0, fixbot_fire_welder},
 	{ai_move2, 0, fixbot_fire_welder},
 	{ai_move2, 0, fixbot_fire_welder},
@@ -1345,7 +1368,7 @@ mmove_t fixbot_move_weld = {
    	NULL
 };
 
-mframe_t fixbot_frames_weld_end[] = {
+static mframe_t fixbot_frames_weld_end[] = {
 	{ai_move2, -2, NULL},
 	{ai_move2, -2, NULL},
 	{ai_move2, -2, NULL},
